@@ -19,7 +19,7 @@ from apycula import codegen
 from apycula import bslib
 from apycula import pindef
 from apycula import fuse_h4x
-from apycula.wirenames import wirenames, clknames
+from apycula.wirenames import wirenames, clknames, wirenumbers, clknumbers
 #TODO proper API
 #from apycula import dat19_h4x
 from apycula import tm_h4x
@@ -118,7 +118,7 @@ params = {
         "package": "UBGA332",
         "device": "GW1N-9C-UBGA332-6",
         "partnumber": "GW1N-LV9UG332C6/I5",
-        "recode_idx": recode_idx_gw1n9, # TODO: recheck
+        "recode_idx": recode_idx_gw1n9,
     },
     "GW1N-4": {
         "package": "PBGA256",
@@ -136,10 +136,12 @@ params = {
         "package": "QFN48",
         "device": "GW1NZ-1-QFN48-6",
         "partnumber": "GW1NZ-LV1QN48C6/I5",
-        "recode_idx": recode_idx_gw1nz_1, # TODO: check
+        "recode_idx": recode_idx_gw1nz_1,
     },
 }[device]
 
+
+
 name_idx = 0
 def make_name(bel, typ):
     global name_idx
@@ -689,9 +691,10 @@ def fse_iologic(fse, db, pin_locations):
                     loc.update(get_shortval(fse, ttyp, _iologic_table[bel_idx], fs))
                 bel.modes.setdefault('ENABLE', loc)
                 # iobuf
-                loc = get_longval(fse, ttyp, _pin_mode_longval[bel_idx],
-                        recode_key(_oddr_io_key))
-                bel.flags.setdefault('IOBUF', loc)
+                # XXX not supporting T/IOBUF yet
+                #loc = get_longval(fse, ttyp, _pin_mode_longval[bel_idx],
+                #        recode_key(_oddr_io_key))
+                #bel.flags.setdefault('IOBUF', loc)
                 bel.portmap = {
                     'D0':  wirenames[dat[f'Iologic{bel_idx}In'][1]],
                     'D1':  wirenames[dat[f'Iologic{bel_idx}In'][2]],
@@ -1116,14 +1119,16 @@ if __name__ == "__main__":
     diff_cap_info = pindef.get_diff_cap_info(device, params['package'], True)
     fse_diff_iob(fse, db, pin_locations, diff_cap_info);
 
-    # diff IOB
-    diff_cap_info = pindef.get_diff_cap_info(device, params['package'], True)
-    fse_diff_iob(fse, db, pin_locations, diff_cap_info);
-
     # bank modes
     fse_banks(fse, db, corners)
 
     chipdb.dat_portmap(dat, db)
+    # XXX GW1NR-9 has interesting IOBA pins on the bottom side
+    if device == 'GW1N-9' :
+        loc = locations[52][0]
+        bel = db.grid[loc[0]][loc[1]].bels['IOBA']
+        bel.portmap['XXX_VSS0'] = wirenames[dat[f'IologicAIn'][40]]
+        bel.portmap['XXX_VSS1'] = wirenames[dat[f'IologicAIn'][42]]
     chipdb.dat_aliases(dat, db)
     chipdb.diff2flag(db)
 
